package org.beetl.sql.test;


import com.zaxxer.hikari.HikariDataSource;
import org.beetl.sql.clazz.kit.ClassLoaderKit;
import org.beetl.sql.core.*;
import org.beetl.sql.core.db.MySqlStyle;
import org.beetl.sql.core.nosql.*;
import org.beetl.sql.core.query.Query;
import org.beetl.sql.ext.DebugInterceptor;

import javax.sql.DataSource;

import java.io.InputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

/**
 * 内部测试新功能或者bug用，所有单元测试参考test目录
 * @author xiandafu
 *
 */

public class QuickTest {
    public static void main(String[] args) throws Exception {
        SQLManager sqlManager = getSQLManager();
        UserDao dao = sqlManager.getMapper(UserDao.class);
        dao.query();
//        MyUser user = sqlManager.unique(MyUser.class,1);
    }

    private  static SQLManager getSQLManager(){
        DataSource dataSource = datasource();
        ConnectionSource source = ConnectionSourceHelper.getSingle(dataSource);
        SQLManagerBuilder builder = new SQLManagerBuilder(source);
        builder.setNc(new UnderlinedNameConversion());
        builder.setInters(new Interceptor[]{new DebugInterceptor()});
        builder.setDbStyle(new MySqlStyle());
        builder.setClassLoaderKit(new ClassLoaderKit(QuickTest.class.getClassLoader()));
        SQLManager sqlManager = builder.build();
        return sqlManager;
    }


    private static   DataSource datasource() {
        HikariDataSource ds = new HikariDataSource();
        ds.setJdbcUrl("jdbc:h2:mem:dbtest;DB_CLOSE_ON_EXIT=FALSE");
        ds.setUsername("sa");
        ds.setPassword("");
        ds.setDriverClassName("org.h2.Driver");
        initData(ds);
        return ds;
    }

    private static void initData(DataSource ds)  {
        Connection conn = null;
        try{
            conn = ds.getConnection();
            String[] sqls = getSqlFromFile();
            for(String sql:sqls ){
                runSql(conn,sql);
            }

        }catch (SQLException sqlException){
            throw new RuntimeException(sqlException);
        }
        finally {
            try {
                if(conn!=null)conn.close();
            } catch (SQLException sqlException) {
                //ignore
            }
        }
    }

    private static String[] getSqlFromFile(){
        try{
            InputStream ins = QuickTest.class.getResourceAsStream("/db/schema.sql");
            int len = ins.available();
            byte[] bs = new byte[len];
            ins.read(bs);
            String str = new String(bs,"UTF-8");
            String[] sql = str.split(";");
            return sql;
        }catch(Exception ex){
            throw new RuntimeException(ex);
        }

    }

    private static void runSql(Connection conn,String sql) throws SQLException {
        PreparedStatement ps = conn.prepareStatement(sql);
        ps.executeUpdate();
        ps.close();
    }


}
